@@ -51,8 +51,6 @@ _PARTED_TABLE_TYPE_RE = re.compile(r'^.*partition\s+table\s*:\s*(gpt|msdos)',
 CONFIGDRIVE_LABEL = "config-2"
 MAX_CONFIG_DRIVE_SIZE_MB = 64
 
-GPT_SIZE_SECTORS = 33
-
 # Maximum disk size supported by MBR is 2TB (2 * 1024 * 1024 MB)
 MAX_DISK_SIZE_MB_SUPPORTED_BY_MBR = 2097152
 
@@ -67,7 +65,7 @@ def list_partitions(device):
     """
     output = utils.execute(
         'parted', '-s', '-m', device, 'unit', 'MiB', 'print',
-        use_standard_locale=True, run_as_root=True)[0]
+        use_standard_locale=True)[0]
     if isinstance(output, bytes):
         output = output.decode("utf-8")
     lines = [line for line in output.split('\n') if line.strip()][2:]
@@ -104,7 +102,7 @@ def count_mbr_partitions(device):
     # -d do not update the kernel table
     # -s print a summary of the partition table
     output, err = utils.execute('partprobe', '-d', '-s', device,
-                                use_standard_locale=True, run_as_root=True)
+                                use_standard_locale=True)
     if 'msdos' not in output:
         raise ValueError('The device %s does not have a valid MBR '
                          'partition table' % device)
@@ -131,8 +129,7 @@ def get_disk_identifier(dev):
     """
     disk_identifier = utils.execute('hexdump', '-s', '440', '-n', '4',
                                     '-e', '''\"0x%08x\"''',
-                                    dev, attempts=5, delay_on_retry=True,
-                                    run_as_root=True)
+                                    dev, attempts=5, delay_on_retry=True)
     return disk_identifier[0]
 
 
@@ -143,7 +140,7 @@ def get_partition_table_type(device):
     :return: dos, gpt or None
     """
     out = utils.execute('parted', '--script', device, '--', 'print',
-                        use_standard_locale=True, run_as_root=True)[0]
+                        use_standard_locale=True)[0]
     m = _PARTED_TABLE_TYPE_RE.search(out)
     if m:
         return m.group(1)
@@ -160,7 +157,7 @@ def _blkid(device, probe=False, fields=None):
         args += sum((['-s', field] for field in fields), [])
 
     output, err = utils.execute('blkid', device, *args,
-                                use_standard_locale=True, run_as_root=True)
+                                use_standard_locale=True)
     if output.strip():
         return output.split(': ', 1)[1]
     else:
@@ -177,7 +174,7 @@ def _lsblk(device, deps=True, fields=None):
         args.append('--output-all')
 
     output, err = utils.execute('lsblk', device, *args,
-                                use_standard_locale=True, run_as_root=True)
+                                use_standard_locale=True)
     return output.strip()
 
 
@@ -435,7 +432,7 @@ def get_and_validate_image_format(filename, ironic_disk_format):
 
     :param filename: The name of the image file to validate.
     :param ironic_disk_format: The ironic-provided expected format of the image
-    :returns: tuple of validated img_format and size
+    :returns: tuple of validated img_format (str) and size (int)
     """
     if CONF.disable_deep_image_inspection:
         data = qemu_img.image_info(filename)
@@ -502,11 +499,16 @@ def block_uuid(dev):
         return info.get('PARTUUID', '')
 
 
-def get_dev_block_size(dev):
-    """Get the device size in 512 byte sectors."""
-    block_sz, cmderr = utils.execute('blockdev', '--getsz', dev,
-                                     run_as_root=True)
-    return int(block_sz)
+def get_dev_byte_size(dev):
+    """Get the device size in bytes."""
+    byte_sz, cmderr = utils.execute('blockdev', '--getsize64', dev)
+    return int(byte_sz)
+
+
+def get_dev_sector_size(dev):
+    """Get the device logical sector size in bytes."""
+    sect_sz, cmderr = utils.execute('blockdev', '--getss', dev)
+    return int(sect_sz)
 
 
 def destroy_disk_metadata(dev, node_uuid):
@@ -524,7 +526,7 @@ def destroy_disk_metadata(dev, node_uuid):
               {'node': node_uuid})
     try:
         utils.execute('wipefs', '--force', '--all', dev,
-                      use_standard_locale=True, run_as_root=True)
+                      use_standard_locale=True)
     except processutils.ProcessExecutionError as e:
         with excutils.save_and_reraise_exception() as ctxt:
             # NOTE(zhenguo): Check if --force option is supported for wipefs,
@@ -532,7 +534,7 @@ def destroy_disk_metadata(dev, node_uuid):
             if '--force' in str(e):
                 ctxt.reraise = False
                 utils.execute('wipefs', '--all', dev,
-                              use_standard_locale=True, run_as_root=True)
+                              use_standard_locale=True)
     # NOTE(TheJulia): sgdisk attempts to load and make sense of the
     # partition tables in advance of wiping the partition data.
     # This means when a CRC error is found, sgdisk fails before
@@ -540,27 +542,36 @@ def destroy_disk_metadata(dev, node_uuid):
     # This is the same bug as
     # https://bugs.launchpad.net/ironic-python-agent/+bug/1737556
 
+    sector_size = get_dev_sector_size(dev)
+    # https://uefi.org/specs/UEFI/2.10/05_GUID_Partition_Table_Format.html If
+    # the block size is 512, the First Usable LBA must be greater than or equal
+    # to 34 [...] if the logical block size is 4096, the First Usable LBA must
+    # be greater than or equal to 6
+    if sector_size == 512:
+        gpt_sectors = 33
+    elif sector_size == 4096:
+        gpt_sectors = 5
+
     # Overwrite the Primary GPT, catch very small partitions (like EBRs)
+    dd_bs = 'bs=%s' % sector_size
     dd_device = 'of=%s' % dev
-    dd_count = 'count=%s' % GPT_SIZE_SECTORS
-    dev_size = get_dev_block_size(dev)
-    if dev_size < GPT_SIZE_SECTORS:
-        dd_count = 'count=%s' % dev_size
-    utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, dd_count,
-                  'oflag=direct', use_standard_locale=True, run_as_root=True)
+    dd_count = 'count=%s' % gpt_sectors
+    dev_size = get_dev_byte_size(dev)
+    if dev_size < gpt_sectors * sector_size:
+        dd_count = 'count=%s' % int(dev_size / sector_size)
+    utils.execute('dd', dd_bs, 'if=/dev/zero', dd_device, dd_count,
+                  'oflag=direct', use_standard_locale=True)
 
     # Overwrite the Secondary GPT, do this only if there could be one
-    if dev_size > GPT_SIZE_SECTORS:
-        gpt_backup = dev_size - GPT_SIZE_SECTORS
+    if dev_size > gpt_sectors * sector_size:
+        gpt_backup = int(dev_size / sector_size - gpt_sectors)
         dd_seek = 'seek=%i' % gpt_backup
-        dd_count = 'count=%s' % GPT_SIZE_SECTORS
-        utils.execute('dd', 'bs=512', 'if=/dev/zero', dd_device, dd_count,
-                      'oflag=direct', dd_seek, use_standard_locale=True,
-                      run_as_root=True)
+        dd_count = 'count=%s' % gpt_sectors
+        utils.execute('dd', dd_bs, 'if=/dev/zero', dd_device, dd_count,
+                      'oflag=direct', dd_seek, use_standard_locale=True)
 
     # Go ahead and let sgdisk run as well.
-    utils.execute('sgdisk', '-Z', dev, use_standard_locale=True,
-                  run_as_root=True)
+    utils.execute('sgdisk', '-Z', dev, use_standard_locale=True)
 
     try:
         wait_for_disk_to_become_available(dev)
@@ -583,11 +594,11 @@ def _fix_gpt_structs(device, node_uuid):
         commands fail.
     """
     try:
-        output, _err = utils.execute('sgdisk', '-v', device, run_as_root=True)
+        output, _err = utils.execute('sgdisk', '-v', device)
 
         search_str = "it doesn't reside\nat the end of the disk"
         if search_str in output:
-            utils.execute('sgdisk', '-e', device, run_as_root=True)
+            utils.execute('sgdisk', '-e', device)
     except (processutils.UnknownArgumentError,
             processutils.ProcessExecutionError, OSError) as e:
         msg = (_('Failed to fix GPT data structures on disk %(disk)s '
@@ -651,8 +662,7 @@ def partprobe(device, attempts=None):
         attempts = CONF.disk_utils.partprobe_attempts
 
     try:
-        utils.execute('partprobe', device, run_as_root=True,
-                      attempts=attempts)
+        utils.execute('partprobe', device, attempts=attempts)
     except (processutils.UnknownArgumentError,
             processutils.ProcessExecutionError, OSError) as e:
         LOG.warning("Unable to probe for partitions on device %(device)s, "
@@ -694,7 +704,7 @@ def trigger_device_rescan(device, attempts=None):
     udev_settle()
     try:
         # Also verify that the partitioning is correct now.
-        utils.execute('sgdisk', '-v', device, run_as_root=True)
+        utils.execute('sgdisk', '-v', device)
     except processutils.ProcessExecutionError as exc:
         LOG.warning('Failed to verify partition tables on device %(dev)s: '
                     '%(err)s', {'dev': device, 'err': exc})
@@ -750,8 +760,7 @@ def wait_for_disk_to_become_available(device):
             # NOTE(TheJuia): fuser outputs a list of found PIDs to stdout.
             #                All other text is returned via stderr, and the
             #                output to a terminal is merged as a result.
-            out, err = utils.execute('fuser', device, check_exit_code=[0, 1],
-                                     run_as_root=True)
+            out, err = utils.execute('fuser', device, check_exit_code=[0, 1])
 
             if not out and not err:
                 return True
